<?php

/**
 * Name: Project.php [PHP class definition]
 * Author: R. Joseph Newton
 * Class Provided: Project
 * Purpose: Provides a Project class for use in project management applications
 * Last Modified: March 29, 2007
 */

include_once('project_defs.php');
include_once('utilities.php');
include_once('MySQLAdapterProjectManager.php');
include_once('Person.php');
include_once('Expenditure.php');
include_once('Allocation.php');
include_once('Milestone.php');
include_once('Deliverable.php');

class Project {
   private $_project_space_guid;
   private $_project_guid;
   private $_name;
   private $_description;
   
   private $_start_date;
   private $_projected_finish;
   private $_finish_date;
   
   private $_lead;
   private $_co_lead;
   private $_participants;
   
   private $_task_headers;
   private $_allocations;
   private $_expenditures;
   
   private $_milestones;
   private $_deliverables;
   
   protected $_db_adapter;
   private $_modified;
   
   public function Project($project_space_guid = 0, $name, $description, $lead, $co_lead = 0, $start_date = array(),
    $projected_finish = array(), $finish_date = array()) {
      $this->_db_adapter = new MySQLAdapterProjectManager();
      
      /* MANDATORY ATTRIBUTES */
      $this->_project_space_guid = $project_space_guid;
      $this->_name = $name;
      $this->_description = $description;
      $this->_lead = $lead;
      
      /* OPTIONAL ATTRIBUTES */
      $this->_co_lead = $co_lead;
      $this->_init_dates($start_date, $projected_finish, $finish_date);
      
      /* MODIFIED FLAG */
      $this->_modified = true;
      
      /* EMPTY CONTAINERS */
      $this->_participants = array();
      $this->_task_headers = array();
      $this->_allocations = array();
      $this->_expenditures = array();
      $this->_milestones = array();
      $this->_deliverables = array();
   }
   
   private function _init_dates($start_date, $projected_finish, $finish_date) {
      $this->_start_date = $start_date;
      $this->_projected_finish = $projected_finish;
      $this->_finish_date = $finish_date;
   }
   
   public function get_by_project_id($project_guid) {
      $adapter = new MySQLAdapterProjectManager();
      $project = NULL;
      $query = "select * from project where project_guid = $project_guid";
      $result = $adapter->db_get_first($query);
      $co_chair = isset($result['co_lead']) ? $result['co_lead'] : 0;
      $start_date = date_ISO_to_array($result['start_date']);
      $projected_finish = date_ISO_to_array($result['projected_finish']);
      $finish_date = date_ISO_to_array($result['finish_date']);
      $project = new Project($result['project_space_guid'], $result['name'],
       $result['description'], $result['lead'], $result['co_lead'],
       $start_date, $projected_finish,  $finish_date);
      $project->_project_guid = $project_guid;
      $project->_modified = false;
      return $project;
   }
   
   public function load_containers() {
      $this->_load_task_headers();
      $this->_load_participants();
      $this->_load_allocations();
      $this->_load_expenditures();
      $this->_load_milestones();
      $this->_load_deliverables();
   }
   
   private function _load_task_headers() {
      $adapter = $this->_db_adapter;
      $task_headers = array();
      $query = 'select task.task_guid, project.name from task join project' .
      ' on task.task_guid = project.project_guid' .
      " where task.parent_project = $this->_project_guid";
      $tasks = $adapter->db_get_all($query);
      foreach ($tasks as $task_header) {
         $this->_task_headers[] = $task_header;
      }
   }
   
   private function _load_participants() {
      $participant_ids = $this->_get_participant_ids();
      foreach ($participant_ids as  $array) {
         if ( !is_array($array) ) {
            return false;
         }
         $participant = Person::get_by_person_id($array['person_guid']);
         $this->_participants[] = $participant;
      }
   }
   
   private function _get_participant_ids() {
      $adapter = $this->_db_adapter;
      
      $query = 'select participant.person_guid from participant join person' .
      ' on participant.person_guid = person.person_guid' .
      " where domain_guid = $this->_project_guid";
      $result = $adapter->db_get_all($query);
      if ( !is_array($result) ) {
         return false;
      }
      return $result;
   }
   
   private function _load_allocations() {
      $allocations = $this->_load_contained_objects('allocation');
      foreach ($allocations as  $array) {
         if ( !is_array($array) ) {
            return false;
         }
         $allocation = Allocation::load_from_array($array);
         $this->_allocations[] = $allocation;
      }
   }
   
   private function _load_expenditures() {
      $expenditures = $this->_load_contained_objects('expenditure');
      foreach ($expenditures as  $array) {
         if ( !is_array($array) ) {
            return false;
         }
         $expenditure = Expenditure::load_from_array($array);
         $this->_expenditures[] = $expenditure;
      }
   }
   
   private function _load_milestones() {
      $milestones = $this->_load_contained_objects('milestone');
      foreach ($milestones as  $array) {
         if ( !is_array($array) ) {
            return false;
         }
         $milestone = Milestone::load_from_array($array);
         $this->_milestones[] = $milestone;
      }
   }
   
   private function _load_deliverables() {
      $deliverables = $this->_load_contained_objects('deliverable');
      foreach ($deliverables as  $array) {
         if ( !is_array($array) ) {
            return false;
         }
         $deliverable = Deliverable::load_from_array($array);
         $this->_deliverables[] = $deliverable;
      }
   }
   
   private function _load_contained_objects($object_name) {
      $adapter = $this->_db_adapter;
      if ( !isset($this->_project_guid) ) {
         return;
      }
      $query = "select * from $object_name where project_guid = $this->_project_guid";
      $objects = $adapter->db_get_all($query);
      if ( !is_array($objects) ) {
         return array();
      }
      return $objects;
   }
   
   public function save() {
      if ( isset($this->_project_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->_modified = false;
      }
      return $result;
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      $project_name = $adapter->db_escape($this->_name);
      $description = $adapter->db_escape($this->_description);
      $co_lead = isset($this->_co_lead) ? $this->_co_lead : 'NULL';
      $start = $this->_start_date;
      $start_string = date_array_to_ISO($start);
      $projected_finish = $this->_projected_finish;
      $projected_finish_string = isset($projected_finish) ?
       date_array_to_ISO($projected_finish) : 'NULL';
      $finish = $this->_finish_date;
      $finish_string = isset($finish) ? date_array_to_ISO($finish) : 'NULL';
      $command = "update project set name = '$project_name', description = '$description'," .
      " lead = $this->_lead, co_lead = $this->_co_lead, start_date = $start_string," .
      " projected_finish = $projected_finish_string, finish_date = $finish_string";
      $command .= " where project_guid = $this->_project_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('project');
      if (!$guid) {
         return false;
      }
      $this->_project_guid = $guid;
      $query = $this->_insert_project_query($guid);
      $result = $adapter->db_command($query);
      if ( isset($this->_co_lead) && $this->_co_lead) {
         $this->_add_participant_by_role($this->_co_lead, 'co_lead');
      }
      if (is_array($result) ) {
         print "project creation error:<br>\n";
        /* dump_array_to_page($result); */
      }
      $this->_add_participant_by_role($this->_lead, 'lead');
      return $result;
   }
   
   private function _insert_project_query($guid) {
      $adapter = $this->_db_adapter;
      $project_name = $adapter->db_escape($this->_name);
      $description = $adapter->db_escape($this->_description);
      $co_lead = isset($this->_co_lead) ? $this->_co_lead : 'NULL';
      $start = $this->_start_date;
      $start_string = date_array_to_ISO($start);
      $projected_finish = $this->_projected_finish;
      $projected_finish_string = isset($projected_finish['year']) ?
       date_array_to_ISO($projected_finish) : 'NULL';
      $finish = $this->_finish_date;
      $finish_string = isset($finish['year']) ? date_array_to_ISO($finish) : 'NULL';
      $query = "insert into project values ($this->_project_guid," .
      " $this->_project_space_guid, '$project_name', '$description'," .
      " $this->_lead, $co_lead, $start_string, $projected_finish_string," .
      " $finish_string)";
      return $query;
   }
   
   private function _add_participant_by_role($person_id, $role_name) {
      $adapter = $this->_db_adapter;
      
      $guid = $this->_project_guid;
      $role_id = $this->_get_role_id($role_name);
      $command = "insert into participant values ($person_id, $guid, $role_id)";
      /*  print "Command is $command<br>\n";  */
      $result = $adapter->db_command($command);
      if ( is_array($result) ) {
         dump_array_to_page($result);
      }
   }

   private function _get_role_id($role_name) {
      $adapter = $this->_db_adapter;
      
      $query = "select role_guid from role where role_name = '" . $role_name . "' and scope = 'project'";
/*      print "role id query is $query<br>\n";   */
      $result = $adapter->db_get_first($query);
/*      print "result seen is " . $result['role_guid'] . "<br />\n"  */
      return $result['role_guid'];
   }
   
   public function get_project_id() {
      return $this->_project_guid;
   }
   
   public function get_project_name() {
      return $this->_name;
   }
   
   public function change_project_name($new_project_name) {
      $this->_name = $new_project_name;
      $this->_modified = true;
   }
   
   public function get_project_lead() {
      return $this->_lead;
   }
   
   public function change_project_lead($new_lead_id) {
      $this->_lead = $new_lead_id;
      $this->_modified = true;
   }
   
   public function get_project_co_lead() {
      return $this->_co_lead;
   }
   
   public function set_project_co_lead($co_lead_id) {
      $this->_co_lead = $co_lead_id;
      $this->_modified = true;
   }
   
   public function get_project_description() {
      return $this->_description;
   }
   
   public function change_project_description($new_description) {
      $this->_description = $new_description;
      $this->_modified = true;
   }
   
   public function get_start_date() {
      return $this->_start_date;
   }
   
   public function set_start_date(array $start_date) {
      $this->_start_date = $start_date;
      $this->_modified = true;
   }
   
   public function get_projected_finish() {
      return $this->_projected_finish;
   }
   
   public function set_projected_finish(array $projected_finish) {
      $this->_projected_finish = $projected_finish;
      $this->_modified = true;
   }
   
   public function get_finish_date() {
      return $this->_finish_date;
   }
   
   public function set_finish_date(array $finish_date) {
      $this->_finish_date = $finish_date;
      $this->_modified = true;
   }
   
   public function is_modified() {
      return $this->_modified;
   }
   
   public function as_array() {
      $project_array = array('project_name' => $this->_name, 'description' => $this->_description,
       'lead' => $this->_lead);
      if ( isset($this->_co_lead) ) {
         $project_array[] = array('co_lead' => $this->_co_lead);
      }
      if ( isset($this->_start_date['year']) ) {
         $project_array[] = array('start_date' => $this->_start_date);
      }
      if ( isset($this->_projected_finish['year']) ) {
         $project_array[] = array('projected_finish' => $this->_projected_finish);
      }
      if ( isset($this->_finish_date['year']) ) {
         $project_array[] = array('finish_date' => $this->_finish_date);
      }
      $project_array[] = array('participants' => $this->_participants);
      $project_array[] = array('task_ids' => $this->_task_ids);
      $project_array[] = array('expenditures' => $this->_expenditures);
      $project_array[] = array('allocations' => $this->_allocations);
      $project_array[] = array('milestones' => $this->_milestones);
      $project_array[] = array('deliverables' => $this->_deliverables);
      return $project_array;
   }
   
   public function to_string() {
      $project_guid = isset($this->_project_guid) ? $this->_project_guid : 'undefined';
      $string = "$this->_project_space_guid :: $project_guid:  $this->_name\n" .
      "$this->_description\n" .
      "Lead: $this->_lead\n";
      if ($this->_co_lead) {
         $string .= "Co-lead: $this->_co_lead\n";
      }
      $start_date = isset($this->_start_date['year']) ? join('-', $this->_start_date) : 'Not set';
      $string .= "Start date: $start_date\n";
      $projected_finish = isset($this->_projected_finish['year']) ? join('-', $this->_projected_finish) : 'Not set';
      $string .= "Projected finish date: $projected_finish\n";
      $finish_date = isset($this->_finish_date['year']) ? join('-', $this->_finish_date) : 'Not set';
      $string .= "Finish date: $finish_date\n";
      return $string;
   }
   
   public function to_HTML_string() {
      $project_guid = isset($this->_project_guid) ? $this->_project_guid : 'ID not set';
      $string = "<p>$this->_project_space_guid :: $project_guid:  $this->_name<br />\n" .
      "$this->_description<br />\n" .
      "Lead: $this->_lead<br />\n";
      if ($this->_co_lead) {
         $string .= "Co-lead: $this->_co_lead<br />\n";
      }
      $start_date = isset($this->_start_date['year']) ? join('-', $this->_start_date) : 'Not set';
      $string .= "Start date: $start_date<br />\n";
      $projected_finish = isset($this->_projected_finish['year']) ? join('-', $this->_projected_finish) : 'Not set';
      $string .= "Projected finish date: $projected_finish<br />\n";
      $finish_date = isset($this->_finish_date['year']) ? join('-', $this->_finish_date) : 'Not set';
      $string .= "Finish date: $finish_date</p>\n";
      return $string;
   }
   
   public function get_participants() {
      return $this->_participants;
   }
   
   public function get_allocations() {
      return $this->_allocations;
   }
   
   public function get_expenditures() {
      return $this->_expenditures;
   }
   
   public function get_milestones() {
      return $this->_milestones;
   }
   
   public function get_deliverables() {
      return $this->_deliverables;
   }
}
?>
